SystemOrganization addCategory: #'Satellogic-Structure'!
SystemOrganization addCategory: #'Satellogic-DataTypes'!
SystemOrganization addCategory: #'Satellogic-Tests'!

TestCase subclass: #SatellogicTest
	instanceVariableNames: 'testObjectFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Tests'!

!SatellogicTest methodsFor: 'private' stamp: 'TomasRodriguez 10/30/2013 09:02'!
setUp

	testObjectFactory := ObjectFactory new .! !

!SatellogicTest methodsFor: 'private' stamp: 'TomasRodriguez 10/29/2013 21:38'!
should: aBlock raiseErrorWithMessage: aMessage

	self should: aBlock  
	raise:  (Error but: MessageNotUnderstood )
	withExceptionDo: [ :anError | self assert: anError messageText = aMessage ]! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/29/2013 21:02'!
test001AnAnswerParameterHasANameAndADataType

	| dataType answerParameter |
	dataType := Unsigned8BitInteger new .
	answerParameter := AnswerParameter of: Unsigned8BitInteger new named: 'battery_temp' .
	
	self assert: answerParameter name = 'battery_temp' .
	self assert: answerParameter dataType = dataType .! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/29/2013 21:39'!
test002CanNotCreateAnAnswerParameterWithAnInvalidName

	self should: [AnswerParameter of: Unsigned8BitInteger new named: '' ] 
	raiseErrorWithMessage: 'The answer parameter must have a name'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/29/2013 21:39'!
test003CanNotCreateAnAnswerParameterWithADataTypeThatDoesNotBelongToTheSatelliteDataTypes

	self should: [AnswerParameter of: String new named: 'un string' ]
	raiseErrorWithMessage: 'The data type does not belong to the satellite'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/29/2013 21:32'!
test004AnStructIsACompositionOfAnswerParameters


	| otherAnswerParameter struct anAnswerParameter |
	anAnswerParameter := AnswerParameter of: Unsigned8BitInteger new named: 'a number' .
	otherAnswerParameter := AnswerParameter of: SatelliteString new named: 'a string' .
	struct := Struct with: (OrderedCollection with: anAnswerParameter with: otherAnswerParameter) .
	
	self assert: struct parameters size = 2 .
	self assert: struct parameters first = anAnswerParameter .
	self assert: struct parameters second = otherAnswerParameter ! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/29/2013 21:41'!
test005CanNotCreateAStructWithLessThan2Parameters


	| otherAnswerParameter struct anAnswerParameter |
	anAnswerParameter := AnswerParameter of: Unsigned8BitInteger new named: 'a number' .
	
	self should: [Struct with: (OrderedCollection with: anAnswerParameter)] 
		raiseErrorWithMessage: 'A struct must have at least 2 parameters'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/29/2013 21:46'!
test006ACommandDefinitionHasAnIdANameACollectionOfInputParametersAndACommandAnswerDefinition

	
	| answer commandDefinition |
	answer := AnswerParameter of: Unsigned8BitInteger new named: 'un numero'.
	commandDefinition := CommandDefinition withId: 5 name: 'name' input: #() answer: answer  .
	
	self assert: commandDefinition id = 5 .
	self assert: commandDefinition name = 'name' .
	self assert: commandDefinition input isEmpty .
	self assert: commandDefinition answer = answer ! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/29/2013 21:50'!
test007CanNotCreateACommandWithAnAnswerThatIsNotACommandAnswerDefinition

	self should: [CommandDefinition withId: 5 name: 'name' input: #() answer: Unsigned8BitInteger new] 
	raiseErrorWithMessage: 'Invalid answer for the command'! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 08:49'!
test008CanNotCreateAStructWithOtherThanAnswerParameters

	| otherAnswerParameter struct anAnswerParameter |
	anAnswerParameter := AnswerParameter of: Unsigned8BitInteger new named: 'a number' .
	
	self should: [Struct with: (OrderedCollection with: anAnswerParameter with: Void new)] 
	raiseErrorWithMessage: 'A Struct can only contain answer parameters from the satellite'! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 08:58'!
test009CanNotCreateACommandDefinitionWithAnInputThatDoesNotIncludeSatelliteDataTypes

	self should: [CommandDefinition withId: 5 name: 'name' input: (Array with: SatelliteString new with: Void new) answer: Void new] 
	raiseErrorWithMessage: 'Invalid input for the command'! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 10:28'!
test010ASubSystemHasAnIdANameAnKnowsTheCommandsThatCanAnswer

	| commands subSystem |
	commands := testObjectFactory validThermalCommands .
	subSystem := SubSystemDefinition withId: 3 name: #THERMAL definingCommands: commands .
	
	self assert: subSystem id = 3 .
	self assert: subSystem name = #THERMAL .
	self assert: subSystem commands size = commands size .
	self assert: (subSystem commands includesAll: commands )! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 10:36'!
test012CanNotCreateASubSystemWithTwoCommandsWithTheSameNameOrId

	| commandsSameName commandsSameId |
	commandsSameName := testObjectFactory validThermalCommands .
	commandsSameName add: (CommandDefinition withId: 50 name: #name input: #() answer: Void new ) .
	commandsSameId := testObjectFactory validThermalCommands .
	commandsSameId add: (CommandDefinition withId: 0 name: #new input: #() answer: Void new ) .
	
	self should: [SubSystemDefinition withId: 3 name: #THERMAL definingCommands: commandsSameName ] 
	raiseErrorWithMessage: 'Cannot have two commands with the same name or id' .
	
	self should: [SubSystemDefinition withId: 3 name: #THERMAL definingCommands: commandsSameId ] 
	raiseErrorWithMessage: 'Cannot have two commands with the same name or id'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 10:30'!
test013ASatelliteIsACompositionOfSubSystems

	| subSystems satellite |
	subSystems := testObjectFactory validSubSystemsForASatellite .
	satellite := SatelliteDefinition composedOf: subSystems .
	
	self assert: satellite subSystems size = 2 .
	self assert: (satellite subSystems includesAll: subSystems )! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 10:31'!
test014CanNotCreateASatelliteWithSubSystemsWhichDoesNotDefineTheBasicCommands

	| subSystems |
	subSystems := OrderedCollection with: (testObjectFactory subSystemWithoutBasiCommands) .

	self should: [SatelliteDefinition composedOf: subSystems ] 
	raiseErrorWithMessage: 'Cannot create a satellite with subsystems which does not define the basic commands: "name", "telemetry", "short_telemetry"'
! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:02'!
test015ACommandCallIsFormedByASatelliteASubSystemACommandAndTheInputParmetersToBeSent

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := satellite subSystems first .
	command := subSystem command: #name .
	commandCall := CommandCall for: command from: subSystem in: satellite withInputParameters: #() .
	
	self assert: commandCall satellite = satellite .
	self assert: commandCall subSystem = subSystem .
	self assert: commandCall command = command .
	self assert: commandCall inputForTheCommand = #() .! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:06'!
test016CanNotCreateACommandCallIfTheSubSystemDoesNotBelongToTheSatellite

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := SubSystemDefinition 
		withId: 10 
		name: #unknown 
		definingCommands: (OrderedCollection with: testObjectFactory aNameCommandDefinition ).
	command := subSystem command: #name .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: #() ]
	raiseErrorWithMessage: 'The subsystem does not belong to the satellite'
! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:11'!
test017CanNotCreateACommandCallIfTheCommandDoesNotBelongToTheSubSystem

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := satellite subSystems first .
	command := CommandDefinition withId: 23 name: #unknown input: #() answer: Void new .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: #() ]
	raiseErrorWithMessage: 'The subsystem cannot answer the command'
! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:54'!
test018CanNotCreateACommandCallIfTheInputParametersSizeIsNotTheSameThatTheCommandDefinitionInputSize

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := satellite subSystems first .
	command := subSystem command: #name .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: #(23) ]
	raiseErrorWithMessage: 'The input parameter does not have the correct number of arguments for the command'
! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:32'!
test019AnUnsignedIntegerCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| unsignedIntegers |
	unsignedIntegers := OrderedCollection new .
	unsignedIntegers 
		add: Unsigned8BitInteger new;
		add: Unsigned16BitInteger new;
		add: Unsigned24BitInteger new;
		add: Unsigned32BitInteger new;
		add: Unsigned64BitInteger new;
		yourself .
	
	unsignedIntegers do: [ :anUnsignedInteger | | exponent |
		exponent := anUnsignedInteger bitsUsed .
		self assert: (anUnsignedInteger canBeRepresentedWith: 1) .
		self deny: (anUnsignedInteger canBeRepresentedWith: -1) .
		self deny: (anUnsignedInteger canBeRepresentedWith: (2 raisedTo: exponent ) ) .
		self deny: (anUnsignedInteger canBeRepresentedWith: 'hola')
		]
! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:34'!
test020ASignedIntegerCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| signedIntegers |
	signedIntegers := OrderedCollection new .
	signedIntegers 
		add: Signed8BitInteger new;
		add: Signed16BitInteger new;
		add: Signed32BitInteger new;
		add: Signed64BitInteger new;
		yourself .
	
	signedIntegers do: [ :aSignedInteger | | exponent |
		exponent := aSignedInteger bitsUsed - 1 .
		self assert: (aSignedInteger canBeRepresentedWith: 1) .
		self assert: (aSignedInteger canBeRepresentedWith: -1) .
		self deny: (aSignedInteger canBeRepresentedWith: (2 raisedTo: exponent ) ) .
		self deny: (aSignedInteger canBeRepresentedWith: ((2 raisedTo: exponent ) + 1) negated ) .
		self deny: (aSignedInteger canBeRepresentedWith: 'hola')
		]
! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:38'!
test021ASatelliteStringCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| satelliteString |
	satelliteString := SatelliteString new .
	
	self assert: ( satelliteString canBeRepresentedWith: 'Hola' ) .
	self deny: ( satelliteString canBeRepresentedWith: 872 ) .
! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:43'!
test022ASizedStringCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| sizedStrings |
	sizedStrings := OrderedCollection new .
	sizedStrings 
		add: String8 new;
		add: String16 new;
		yourself .
	
	sizedStrings do: [ :aSizedString | | stringSize |
		
		stringSize := aSizedString sizeRepresentation bitsUsed .
		self assert: (aSizedString canBeRepresentedWith: 'asdasd') .
		self deny: (aSizedString canBeRepresentedWith: 123 ) .
		self deny: (aSizedString canBeRepresentedWith: (String new: (2 raisedTo: stringSize)))
		]! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:48'!
test023AnArrayOfFixedSizeCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| arrayOfFixedSize |
	arrayOfFixedSize := ArrayOfFixedSize of: Unsigned64BitInteger new withSize: 5 .
	
	self assert: (arrayOfFixedSize canBeRepresentedWith: #(2 3 4 5 6) ) .
	self deny: (arrayOfFixedSize canBeRepresentedWith: #(2 3 5 6) ) .
	self deny: (arrayOfFixedSize canBeRepresentedWith: #('hola' 'hola' 'hola' 'hola' 'hola') ) .! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:51'!
test024AnEnumCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| enumeration |
	enumeration := Enum withLabels: #(ON OFF).
	
	self assert: (enumeration canBeRepresentedWith: #ON ) .
	self assert: (enumeration canBeRepresentedWith: #OFF ) .
	self deny: (enumeration canBeRepresentedWith: #UNKNOWN ) .! !

!SatellogicTest methodsFor: 'Tests 02 CommandCalls' stamp: 'TomasRodriguez 10/30/2013 11:59'!
test025CanNotCreateACommandCallIfTheInputParametersDoesNotFitTheCommandDefinitionInput

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := satellite subSystems first .
	command := subSystem command: #setMatrixKeyKey: .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: #('hola') ]
	raiseErrorWithMessage: 'The input parameter does not fit the command input' .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: (Array with: #(23)) ]
	raiseErrorWithMessage: 'The input parameter does not fit the command input' .
	
	CommandCall for: command from: subSystem in: satellite withInputParameters: #(23)
! !

Object subclass: #CommandAnswerDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

CommandAnswerDefinition subclass: #AnswerParameter
	instanceVariableNames: 'name dataType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!AnswerParameter class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:10'!
invalidDataTypeMessage
	
	^ 'The data type does not belong to the satellite'! !

!AnswerParameter class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:07'!
invalidNameMessage
	
	^ 'The answer parameter must have a name'! !

!AnswerParameter class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:10'!
of: aDataType named: aString 
	
	(aDataType isKindOf: SatelliteDataType) ifFalse: [ self error: self invalidDataTypeMessage ] .
	aString isEmpty ifTrue: [ self error: self invalidNameMessage ] .
	
	^ self new initializeWithDataType: aDataType andName: aString ! !

!AnswerParameter methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:00'!
dataType 

	^ dataType ! !

!AnswerParameter methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:00'!
initializeWithDataType: aDataType andName: aString 

	name := aString .
	dataType := aDataType ! !

!AnswerParameter methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:00'!
name

	^ name! !

CommandAnswerDefinition subclass: #ArrayUpToEndOfPackage
	instanceVariableNames: 'dataType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!ArrayUpToEndOfPackage class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 21:14'!
of: aDataType 

	^ self new initializeWithDataType: aDataType ! !

!ArrayUpToEndOfPackage methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/19/2013 10:22'!
dataType

	^ dataType! !

!ArrayUpToEndOfPackage methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/19/2013 10:21'!
initializeWithDataType: aDataType 

	dataType := aDataType ! !

CommandAnswerDefinition subclass: #Struct
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!Struct class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 08:50'!
invalidAnswerParameterMessage
	
	^ 'A Struct can only contain answer parameters from the satellite'! !

!Struct class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 08:51'!
invalidParameterSizeMessage
	
	^ 'A struct must have at least 2 parameters'! !

!Struct class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 08:51'!
with: anOrderedCollection 
	
	anOrderedCollection size < 2 ifTrue: [ self error: self invalidParameterSizeMessage ]. 
	(anOrderedCollection allSatisfy: [ :anElement | anElement isKindOf: AnswerParameter ])
		ifFalse: [ self error: self invalidAnswerParameterMessage ].
	
	^ self new initializeWithAnswerParameters: anOrderedCollection ! !

!Struct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:32'!
initializeWithAnswerParameters: anOrderedCollection 
	
	parameters := anOrderedCollection ! !

!Struct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:32'!
parameters

	^ parameters! !

CommandAnswerDefinition subclass: #Void
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

Object subclass: #CommandCall
	instanceVariableNames: 'command subSystem satellite inputForTheCommand'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Tests'!

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:12'!
commandMissingMessage
	
	^ 'The subsystem cannot answer the command'! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:00'!
for: aCommandDefinition from: aSubSystemDefinition in: aSatelliteDefinition withInputParameters: anOrderedCollection

	self verifyThat: aSubSystemDefinition isComposedIn: aSatelliteDefinition.
	self verifyThat: aCommandDefinition isDefinedIn: aSubSystemDefinition.
	self verifyThat: anOrderedCollection fitsTheInputOf: aCommandDefinition.
	
	^ self new
		initializeWithCommandDefinition: aCommandDefinition
		withSubSystemDefinition: aSubSystemDefinition
		withSatelliteDefinition: aSatelliteDefinition
		withInputForTheCommand: anOrderedCollection! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:58'!
invalidInputMessage
	
	^ 'The input parameter does not fit the command input'! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:21'!
invalidNumberOfInputsMessage
	
	^ 'The input parameter does not have the correct number of arguments for the command'! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:08'!
subSystemMissingMessage
	
	^ 'The subsystem does not belong to the satellite'! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:02'!
verifyThat: aCommandInput fitsTheInputOf: aCommandDefinition
	
	| stream |		
	stream := ReadStream on: aCommandInput .
	
	(aCommandDefinition input allSatisfy: [ :anInput | anInput canBeRepresentedWith: stream next ])
		ifFalse: [ self error: self invalidInputMessage ] .
	
	stream atEnd ifFalse: [ self error: self invalidNumberOfInputsMessage ].! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:14'!
verifyThat: aSubSystemDefinition isComposedIn: aSatelliteDefinition
	(aSatelliteDefinition subSystems includes: aSubSystemDefinition)
		ifFalse: [ self error: self subSystemMissingMessage ]! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:15'!
verifyThat: aCommandDefinition isDefinedIn: aSubSystemDefinition
	(aSubSystemDefinition commands includes: aCommandDefinition)
		ifFalse: [ self error: self commandMissingMessage ]! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:03'!
command

	^ command! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:02'!
initializeWithCommandDefinition: aCommandDefinition withSubSystemDefinition: aSubSystemDefinition withSatelliteDefinition: aSatelliteDefinition withInputForTheCommand: anOrderedCollection 

	command := aCommandDefinition .
	subSystem := aSubSystemDefinition .
	satellite := aSatelliteDefinition .
	inputForTheCommand := anOrderedCollection ! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:03'!
inputForTheCommand

	^ inputForTheCommand! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:02'!
satellite

	^ satellite! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:03'!
subSystem

	^ subSystem! !

Object subclass: #IdAndNameIdentifier
	instanceVariableNames: 'id name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!IdAndNameIdentifier class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:45'!
with: anId with: aName 
	
	^ self new initializeWithId: anId withName: aName ! !

!IdAndNameIdentifier methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:46'!
= anIdentifier

	^ id = anIdentifier id or: (name = anIdentifier name)! !

!IdAndNameIdentifier methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:46'!
id

	^ id! !

!IdAndNameIdentifier methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:46'!
initializeWithId: anId withName: aName 

	id := anId .
	name := aName ! !

!IdAndNameIdentifier methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:46'!
name

	^ name! !

Object subclass: #IdentifiedWithIdAndName
	instanceVariableNames: 'identifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

IdentifiedWithIdAndName subclass: #CommandDefinition
	instanceVariableNames: 'input answer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!CommandDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:51'!
invalidAnswerMessage
	
	^ 'Invalid answer for the command'! !

!CommandDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 08:58'!
invalidInputMessage 

	^ 'Invalid input for the command'! !

!CommandDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:08'!
withId: anId name: aName input: anInput answer: aCommandAnswerDefinition 
	
	(aCommandAnswerDefinition isKindOf: CommandAnswerDefinition) 
		ifFalse: [ self error: self invalidAnswerMessage ].
	(anInput allSatisfy: [ :anElement | anElement isKindOf: SatelliteDataType ])
		ifFalse: [ self error: self invalidInputMessage ].
	
	^ self new initializeWithId: anId withName: aName withInput: anInput withAnswer: aCommandAnswerDefinition ! !

!CommandDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:48'!
answer 

	^ answer ! !

!CommandDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:45'!
initializeWithId: anId withName: aName withInput: anOrderedCollection withAnswer: aCommandAnswerDefinition

	self initializeId: anId andName: aName .
	input := anOrderedCollection .
	answer := aCommandAnswerDefinition ! !

!CommandDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:48'!
input

	^ input ! !

!IdentifiedWithIdAndName methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:47'!
id

	^ identifier id! !

!IdentifiedWithIdAndName methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:47'!
identifier 

	^ identifier ! !

!IdentifiedWithIdAndName methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:46'!
initializeId: anId andName: aName 

	identifier := IdAndNameIdentifier with: anId with: aName .! !

!IdentifiedWithIdAndName methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:47'!
name

	^ identifier name! !

IdentifiedWithIdAndName subclass: #SubSystemDefinition
	instanceVariableNames: 'commandsManual'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!SubSystemDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:14'!
withId: anId name: aName definingCommands: aCollectionOfCommandDefinitions

	| commandsManual |
	commandsManual := CommandsManual for: aCollectionOfCommandDefinitions .
	
	^ self new initializeWithId: anId withName: aName withCommands: commandsManual  ! !

!SubSystemDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:58'!
command: aName 
	
	^ commandsManual at: aName ! !

!SubSystemDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:29'!
commands

	^ commandsManual values! !

!SubSystemDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:45'!
initializeWithId: anId withName: aName withCommands: aCollectionOfCommandDefinitions 

	self initializeId: anId andName: aName .
	commandsManual := aCollectionOfCommandDefinitions ! !

Object subclass: #IdentifiedWithIdAndNameDictionary
	instanceVariableNames: 'dictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

IdentifiedWithIdAndNameDictionary subclass: #CommandsManual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!CommandsManual class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:35'!
twoDefinitionsWithTheSameIdentifierMessage
	
	^ 'Cannot have two commands with the same name or id'! !

!IdentifiedWithIdAndNameDictionary class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:33'!
for: aCollectionOfDefinitions 

	(self twoDefinitionsWithTheSameIdentifier: aCollectionOfDefinitions )
		ifTrue: [ self error: self twoDefinitionsWithTheSameIdentifierMessage ] .

	^ self new initializeWithDefinitions: aCollectionOfDefinitions ! !

!IdentifiedWithIdAndNameDictionary class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:47'!
twoDefinitionsWithTheSameIdentifier: aCollectionOfDefinitions
	
	| identifiers |
	identifiers := aCollectionOfDefinitions collect: [ :aDefinition | aDefinition identifier ] .
	^ identifiers anySatisfy: [ :anIdentifier | (identifiers occurrencesOf: anIdentifier) > 1]! !

!IdentifiedWithIdAndNameDictionary class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:21'!
twoDefinitionsWithTheSameIdentifierMessage

	self subclassResponsibility! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:59'!
at: aName

	^ dictionary at: aName ! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:27'!
initializeWithDefinitions: aCollection  
	
	dictionary := Dictionary new .
	aCollection 
		do: [ :anElement | dictionary at: anElement name put: anElement ]! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:27'!
size

	^ dictionary size! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:29'!
values

	^ dictionary values! !

IdentifiedWithIdAndNameDictionary subclass: #SubSystemManual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!SubSystemManual class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:21'!
twoDefinitionsWithTheSameIdentifierMessage
	
	^ 'Cannot have two subsystems with the same name or id'! !

Object subclass: #ObjectFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Tests'!

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:04'!
aNameCommandDefinition
	
	^ CommandDefinition 
		withId: 0 
		name: #name 
		input: #() 
		answer: (AnswerParameter of: SatelliteString new named: #name )! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:06'!
aPlatformResetCommandDefinition
	
	^ CommandDefinition
		withId: 33
		name: #reset 
		input: #()
		answer: Void new! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:06'!
aPlatformSetModeModeCommandDefinition
	
	^ CommandDefinition
		withId: 32
		name: #setModeMode: 
		input: (OrderedCollection with: Unsigned8BitInteger new)
		answer: Void new! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:05'!
aPlatformShortTelemetryCommandDefinition
	
	^ CommandDefinition
		withId: 2
		name: #short_telemetry
		input: #()
		answer: Void new! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:03'!
aPlatformSubSystemDefinition
	
	^ SubSystemDefinition withId: 0 name: #PLATFORM definingCommands: self validPlatformCommands .! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:05'!
aPlatformTelemetryCommandDefinition
	
	| answer |
	answer := OrderedCollection new .
	answer add: (AnswerParameter of: Unsigned64BitInteger new named: 'tickCount' ) .
	answer add: (AnswerParameter of: Unsigned8BitInteger new named: 'currentMode' ) .
	
	^ CommandDefinition
		withId: 1
		name: #telemetry
		input: #()
		answer: (Struct with: answer)! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:07'!
aPlatformVersionCommandDefinition
	
	^ CommandDefinition
		withId: 34
		name: #version 
		input: #()
		answer: (AnswerParameter of: SatelliteString new named: 'version')! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:59'!
aThermalSetMatrixKeyKeyCommandDefinition
	
	^ CommandDefinition
		withId: 32
		name: #setMatrixKeyKey:
		input: (OrderedCollection with: Unsigned32BitInteger new)
		answer: Void new! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:58'!
aThermalShortTelemetryCommandDefinition
	
	^ CommandDefinition
		withId: 2
		name: #short_telemetry
		input: #()
		answer: (AnswerParameter of: Signed8BitInteger new named: 'battery_temp' )! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:02'!
aThermalSubSystemDefinition
	
	^ SubSystemDefinition withId: 3 name: #THERMAL definingCommands: self validThermalCommands .! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:59'!
aThermalTelemetryCommandDefinition
	
	| answer |
	answer := OrderedCollection new .
	answer add: (AnswerParameter of: Signed16BitInteger new named: 'battery_temp' ) .
	answer add: (AnswerParameter of: Signed16BitInteger new named: 'radio_temp' ) .
	
	^ CommandDefinition
		withId: 1
		name: #telemetry
		input: #()
		answer: (Struct with: answer )! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:32'!
subSystemWithoutBasiCommands
	
	^ SubSystemDefinition withId: 1 name: #invalid definingCommands: #()! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:04'!
validPlatformCommands
	
	| commands |
	commands := OrderedCollection new. 
	^ commands	
		add: self aNameCommandDefinition ;
		add: self aPlatformTelemetryCommandDefinition ;
		add: self aPlatformShortTelemetryCommandDefinition ;
		add: self aPlatformSetModeModeCommandDefinition ;
		add: self aPlatformResetCommandDefinition ;
		add: self aPlatformVersionCommandDefinition ;
		yourself ! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:55'!
validSatelliteDefinition
	
	^ SatelliteDefinition composedOf: self validSubSystemsForASatellite .! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:03'!
validSubSystemsForASatellite
	
	| subSystems |
	subSystems := OrderedCollection new. 
	^ subSystems	
		add: self aThermalSubSystemDefinition ;
		add: self aPlatformSubSystemDefinition ;
		yourself ! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:59'!
validThermalCommands
	
	| commands |
	commands := OrderedCollection new. 
	^ commands	
		add: self aNameCommandDefinition ;
		add: self aThermalTelemetryCommandDefinition ;
		add: self aThermalShortTelemetryCommandDefinition ;
		add: self aThermalSetMatrixKeyKeyCommandDefinition ;
		yourself ! !

Object subclass: #SatelliteDataType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

SatelliteDataType subclass: #ArrayOfFixedSize
	instanceVariableNames: 'dataType size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!ArrayOfFixedSize class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 21:14'!
of: aSatelliteDataType withSize: aSize 
	
	^ self new initializeWithDataType: aSatelliteDataType andSize: aSize ! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:12'!
accept: aVisitor 
	
	aVisitor visitArrayOfFixedSize: self! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:49'!
canBeRepresentedWith: aCollection

	aCollection isCollection ifFalse: [ ^ false ] .
	^ aCollection size = size and: ( aCollection allSatisfy: [:anElement | dataType canBeRepresentedWith: anElement ])! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:14'!
dataType

	^ dataType! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:11'!
initializeWithDataType: aSatelliteDataType andSize: aSize 
	
	dataType := aSatelliteDataType .
	size := aSize ! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:24'!
size

	^ size! !

SatelliteDataType subclass: #Enum
	instanceVariableNames: 'labels'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Enum class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:01'!
withLabels: anArrayOfLabelds
	
	^ self new initializeWithLabels: anArrayOfLabelds ! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:03'!
accept: aVisitor  
	
	aVisitor visitEnum: self! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:52'!
canBeRepresentedWith: aLabel

	^ labels includes: aLabel ! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:21'!
initializeWithLabels: anArrayOfLabels 
	
	labels := anArrayOfLabels ! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:55'!
labelAt: anInteger

	^ labels at: (anInteger + 1) ifAbsent: [ self error: self valueNotFound ]! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:22'!
labels

	^ labels ! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:23'!
valueAt: aLabel

	^ (labels indexOf: aLabel ifAbsent: [ self error: self labelNotFound ]) -1! !

!SatelliteDataType methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:23'!
= aSatelliteInteger

	^ self class = aSatelliteInteger class! !

!SatelliteDataType methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/19/2013 11:28'!
accept: aVisitor 

	self subclassResponsibility ! !

!SatelliteDataType methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:37'!
canBeRepresentedWith: anInteger

	self subclassResponsibility ! !

SatelliteDataType subclass: #SatelliteInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!SatelliteInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed

	self subclassResponsibility ! !

!SatelliteInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bytes

	^ self bitsUsed / 8! !

SatelliteInteger subclass: #SignedInteger
	instanceVariableNames: 'min max'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

SignedInteger subclass: #Signed16BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Signed16BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 16! !

SignedInteger subclass: #Signed32BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Signed32BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 32! !

SignedInteger subclass: #Signed64BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Signed64BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 64! !

SignedInteger subclass: #Signed8BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Signed8BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 8! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/28/2013 10:22'!
accept: aVisitor 
	
	^ aVisitor visitSignedInteger: self .! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:35'!
canBeRepresentedWith: anInteger

	anInteger isInteger ifFalse: [ ^ false ] .
	^ anInteger between: min and: max! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
initialize

	| bits |
	bits := self bitsUsed .
	min := (2 raisedTo: bits - 1) negated .
	max := (2 raisedTo: bits - 1) - 1! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 20:34'!
max

	^ max! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 20:34'!
min

	^ min! !

SatelliteInteger subclass: #UnsignedInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

UnsignedInteger subclass: #Unsigned16BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned16BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	^ 16! !

UnsignedInteger subclass: #Unsigned24BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned24BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 24! !

UnsignedInteger subclass: #Unsigned32BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned32BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 32! !

UnsignedInteger subclass: #Unsigned64BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned64BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 64! !

UnsignedInteger subclass: #Unsigned8BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned8BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 8! !

!UnsignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/28/2013 09:44'!
accept: aVisitor 
	
	^ aVisitor visitUnsignedInteger: self .! !

!UnsignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:31'!
canBeRepresentedWith: anInteger

	anInteger isInteger ifFalse: [ ^ false ] .
	^ anInteger > 0 and: ( anInteger < (2 raisedTo: self bitsUsed) )! !

SatelliteDataType subclass: #SatelliteString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!SatelliteString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/28/2013 10:22'!
accept: aVisitor 
	
	^ aVisitor visitSatelliteString: self! !

!SatelliteString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:39'!
canBeRepresentedWith: aString

	^ aString isString! !

SatelliteDataType subclass: #SizedString
	instanceVariableNames: 'dataTypeToSerializeSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!SizedString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/28/2013 10:23'!
accept: aVisitor 
	
	^ aVisitor visitStringWithSize: self! !

!SizedString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:44'!
canBeRepresentedWith: aString

	aString isString ifFalse: [ ^ false ] .
	^ aString size < (2 raisedTo: self sizeRepresentation bitsUsed )! !

!SizedString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:19'!
sizeRepresentation
	
	self subclassResponsibility ! !

SizedString subclass: #String16
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!String16 methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:19'!
sizeRepresentation
	
	^ Unsigned16BitInteger new! !

SizedString subclass: #String8
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!String8 methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:19'!
sizeRepresentation
	
	^ Unsigned8BitInteger new! !

Object subclass: #SatelliteDefinition
	instanceVariableNames: 'subSystemManual'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!SatelliteDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:39'!
allSubSystemsDefineTheBasicCommands: aCollectionOfSubSystemsDefinition

	^ aCollectionOfSubSystemsDefinition allSatisfy: [ :aSubSystem | self subSystemDefineBasicCommands: aSubSystem  ]
	 
	
	! !

!SatelliteDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:41'!
basicCommandsMissingMessage
	
	^ 'Cannot create a satellite with subsystems which does not define the basic commands: "name", "telemetry", "short_telemetry"'! !

!SatelliteDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:33'!
composedOf: aCollectionOfSubSystems 
	
	| aSubSystemManual |
	aSubSystemManual := SubSystemManual for: aCollectionOfSubSystems .
	
	(self allSubSystemsDefineTheBasicCommands: aCollectionOfSubSystems ) 
		ifFalse: [ self error: self basicCommandsMissingMessage ] .
	
	^ self new initializeWithSubSystems: aSubSystemManual ! !

!SatelliteDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:39'!
subSystemDefineBasicCommands: aSubSystemDefinition 
	
	|aCollectionOfCommandDefinitions hasTelemetry hasShortTelemetry hasName |
	aCollectionOfCommandDefinitions := aSubSystemDefinition commands .
	
	hasTelemetry := aCollectionOfCommandDefinitions anySatisfy: [ :aCommand | aCommand name = 'telemetry' ] .
	hasShortTelemetry := aCollectionOfCommandDefinitions anySatisfy: [ :aCommand | aCommand name = 'short_telemetry' ] .
	hasName := aCollectionOfCommandDefinitions anySatisfy: [ :aCommand | aCommand name = 'name' ] .
	
	^ hasTelemetry and: (hasShortTelemetry and: (hasName))! !

!SatelliteDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:12'!
initializeWithSubSystems: aSubSystemManual 
	
	subSystemManual := aSubSystemManual ! !

!SatelliteDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:30'!
subSystems

	^ subSystemManual values! !

!SatelliteDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:12'!
subSystemsManual
	
	^ subSystemManual ! !
